package net.sf.jsequnit.diagram;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Logger;

import net.sf.jsequnit.jseq.sdedit.SdeditPngDiagram;
import net.sf.jsequnit.process.ProcessController;
import th.co.edge.jseq.ActivationList;
import th.co.edge.jseq.Diagram;
import th.co.edge.jseq.EventThread;

import com.sun.jdi.VirtualMachine;

/**
 * Generates sequence diagrams based on tracing of test executions.
 * 
 * It uses {@code TestMethodRunnerFactory} to generate a {@code VirtualMachine}.
 * The method calls in this VM are traced similar to a debugger.
 * 
 * @author jacek.ratzinger
 *
 */
public class DiagramCreator {
	
	private Logger log = Logger.getLogger(DiagramCreator.class.getName());

	private static final String[] DEFAULT_EXCLUDES = {
									"java.*",
									"javax.*",
									"$Proxy*",
									"sun.*",
									"com.sun.*"};
	
	private String PROPERTY_DEFAULT_EXCLUDES = "jsequnit.default.excludes";
	
	/**
	 * Creates a sequenced diagram based on the trace of the executed test method.
	 * 
	 * The following sequence diagram shows how this method works.
	 * 
	 * @sequence.diagram
	 *   test=net.sf.jsequnit.diagram.DiagramCreatorTest.testCreateSequenceDiagram
	 *   omit=org.*,th.co.edge.jseq.EventThread,th.co.edge.jseq.EventThread$ThreadTrace,net.sf.jsequnit.process.ProcessController$*,net.sf.sdedit.*,th.co.edge.jseq.ActivationList,th.co.edge.jseq.Activation,net.sf.jsequnit.process.ProcessController
	 *   upto=net.sf.jsequnit.jseq.sdedit.SdeditTextDiagram.save
	 * 
	 * @param qualifiedTestMethod
	 *            The fully qualified name of the test method that should be executed.
	 * @param qualifiedTraceMethod
	 *            The fully qualified name of the method, where the tracing should start.
	 *            This is the first method to appear / is called in the resulting sequence diagram.
	 * @param includes
	 *            A list of packages, classes, and/or methods that should be included in the sequence diagram.
	 *            This is a simplified regular expression
	 *            that must either be an exact match or end with an asterix (*).
	 *            For example foo.Foo or baa.*.
	 *            Multiple exclude patterns are separated by a comma (,).
	 * @param excludes
	 *            A list of packages, classes, and/or methods that should be excluded from the sequence diagram.
	 *            This is a simplified regular expression
	 *            that must either be an exact match or end with an asterix (*).
	 *            For example foo.Foo or baa.*.
	 *            Multiple exclude patterns are separated by a comma (,).
	 * @param boundaryMethods
	 *            A list of methods where tracing should stop.
	 *            In contrast to the exclude parameter,
	 *            where method calls are excluded,
	 *            but subsequent calls of the excluded methods still appear in the sequence diagram,
	 *            the boundary methods are included in the resulting diagram,
	 *            but all its sub calls are omitted.
	 * @param isPublicOnly
	 *            If true only public methods are included in the resulting sequence diagram.
	 * @param outputFileName
	 *            The full path to the image file for the sequence diagram.
	 * @param outputFileName2 
	 * 
	 * @throws NoSuchMethodException
	 *            if the JVM cannot find one of the given methods.
	 * @throws IOException
	 *            if an error occurs while writing the image file.
	 */
	public void createSequenceDiagram(
			String qualifiedTestMethod, 
			String qualifiedTraceMethod, 
			List<String> includes,
			List<String> excludes, 
			List<String> boundaryMethods, 
			boolean isPublicOnly,
			String testExecutionClasspath, 
			String outputFileName)
	throws NoSuchMethodException, IOException {
		VirtualMachine testExecutionVirtualMachine = 
				getTestExecutionVirtualMachine(qualifiedTestMethod, testExecutionClasspath);
		ProcessController processController = redirectStreams(testExecutionVirtualMachine);
		ActivationList activationList = traceMethod(
											testExecutionVirtualMachine, 
											qualifiedTraceMethod,
											includes,
											excludes, 
											boundaryMethods,
											isPublicOnly);
		stopRedirectStreams(processController);
		if (testExecutionVirtualMachine.process().exitValue() != 0) {
			throw new IOException("test " + qualifiedTestMethod + " failed");
		}
		writeSequenceDiagram(activationList, outputFileName);
	}

	private VirtualMachine getTestExecutionVirtualMachine(String qualifiedTestMethodName, String testExecutionClasspath)
	throws NoSuchMethodException {
		TestMethodRunner testMethodRunner = TestMethodRunnerFactory.newTestMethodRunner();
		VirtualMachine debugVm = testMethodRunner.getDebugVm(qualifiedTestMethodName, testExecutionClasspath);
		if (debugVm == null) {
			String errMsg = "could not create debug session (virtual machine image) for " + qualifiedTestMethodName;
			throw new NoSuchMethodException(errMsg);
		}
		return debugVm;
	}
	
	private ActivationList traceMethod(
			VirtualMachine testExecutionVirtualMachine, 
			String tracedMethod, 
			List<String> includes,
			List<String> excludes, 
			List<String> boundaryMethods, 
			boolean isPublicOnly) 
	{
		boolean trace = false;
		List<String> allExcludes = new ArrayList<String>();
		allExcludes.addAll(Arrays.asList(getDefaultExcludes()));
		allExcludes.addAll(excludes);
		ActivationList rootActivations = new ActivationList();
		EventThread eventThread = new EventThread(
										testExecutionVirtualMachine, 
										rootActivations,
										includes,
										allExcludes, 
										boundaryMethods,
										isPublicOnly,
										trace);
        eventThread.setEventRequests(tracedMethod);
        eventThread.start();
        try {
			eventThread.join();
		} catch (InterruptedException ie) {
			log.warning("event recording thread interrupted: " + ie.getMessage());
			log.warning(ie.getStackTrace().toString());
		}
        return rootActivations;
	}

	private String[] getDefaultExcludes() {
		String defaultExcludes = System.getProperty(PROPERTY_DEFAULT_EXCLUDES);
		if (defaultExcludes != null) {
			return defaultExcludes.split(",");
		} else {
			return DEFAULT_EXCLUDES;
		}
	}
	
	private ProcessController redirectStreams(VirtualMachine virtualMachine) {
		Process testExecutionProcess = virtualMachine.process();
		ProcessController processController = new ProcessController(testExecutionProcess);
		processController.startStreamForwarding();
		return processController;
	}
	
	private void stopRedirectStreams(ProcessController processController) {
		processController.markFinished();
	}
	
	private void writeSequenceDiagram(ActivationList activationList, String pngFileName) 
	throws IOException {
		if (activationList == null || activationList.size() == 0) return;
		
		try {
			Diagram pngDiagram = new SdeditPngDiagram(activationList);
			File pngFile = new File(pngFileName);
			pngFile.getParentFile().mkdirs();
			pngDiagram.save(pngFile);
		} catch (Exception e) {
			throw new IOException(e);
		}
	}

}
