package net.sf.jsequnit.javadoc;

import com.sun.javadoc.DocErrorReporter;
import com.sun.javadoc.Doclet;
import com.sun.javadoc.LanguageVersion;
import com.sun.javadoc.RootDoc;
import com.sun.tools.doclets.standard.Standard;

/**
 * A {@code Doclet} to be used with the JavaDoc tool to generate JavaDoc documentation
 * containing sequence diagrams.
 * 
 * To use the doclet it is necessary to pass the JavaDoc executable the following parameters:
 * <ul>
 *   <li>
 *     '-doclet net.sf.jsequnit.javadoc.SequenceDoclet'<br/>
 *     instructs JavaDoc to use the JSeqUnit doclet
 *   </li>
 *   <li>
 *     '-docletpath path/to/the/jsequnit-&lt;version&gt;.jar';path/to/compiled/classes;path/to/compiled/test/classes<br/>
 *     indicates where to find the jar file containing the classes of JSeqUnit.
 *     Additionally, this information is used to setup the environment for the JUnit test execution
 *     to trace method calls and generate sequence diagrams.
 *     The compiled classes of the sequence tags are necessary to execute for the tracing of the method calls.
 *     The JUnit tests provide the environment for the method calls.
 *   </li>
 *   <li>
 *     '-classpath lib/junit-4.10.jar;lib/jseq-0.5.jar;lib/sdedit-light-3.1.jar'<br/>
 *     It is necessary to have JUnit in the classpath,
 *     because the JavaDoc tool requires access to all libraries
 *     from which annotations are used (e.g. @Test from JUnit)
 *   </li>
 * </ul>
 * 
 * @author jacek.ratzinger
 *
 */
public class SequenceDoclet extends Doclet {
	
	private static String testExecutionClasspath;
	
	/* (non-Javadoc)
	 * @see com.sun.javadoc.Doclet#languageVersion()
	 */
	public static LanguageVersion languageVersion() {
		return Standard.languageVersion();
	}
	
	/* (non-Javadoc)
	 * @see com.sun.javadoc.Doclet#validOptions(java.lang.String[][], com.sun.javadoc.DocErrorReporter)
	 */
	public static boolean validOptions(String[][] options, DocErrorReporter reporter) {
		return Standard.validOptions(options, reporter);
	}
	
	/* (non-Javadoc)
	 * @see com.sun.javadoc.Doclet#optionLength(java.lang.String)
	 */
	public static int optionLength(String option) {
		return Standard.optionLength(option);
	}
	
	/**
	 * Generate the documentation. This method is called by the JavaDoc tool.
	 * First read the relevant command line options
	 * and then run the standard doclet
	 * that registers the {@code SequenceDoclet} to handle tags for sequence diagram generation.
	 * 
	 * @param root
	 *            Represents the root of the program structure information for one run of javadoc.
	 * @return True on success.
	 */
	public static boolean start(RootDoc root){
		return runJavaDoc(root);
	}

	private static boolean runJavaDoc(RootDoc rootDoc) {
		String docletpath = getOptionValue(rootDoc.options(), "-docletpath");
		String classpath = getOptionValue(rootDoc.options(), "-classpath");
		String tagletpath = docletpath + ";" + classpath;
		String[][] tagletArgs = {
				{"-taglet", SequenceTaglet.class.getName()},
				{"-tagletpath", tagletpath}};
		WrappedRootDoc wrappedRootDoc = new WrappedRootDoc(rootDoc, tagletArgs);
		return Standard.start(wrappedRootDoc);
	}
	
	private static String getOptionValue(String[][] options, String optionName) {
		String optionValue = null;
		for (String[] option : options) {
			if (optionName.equals(option[0])) {
				optionValue = option[1];
			}
		}
		return optionValue;
	}

}
