package net.sf.jsequnit.jseq.sdedit;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import th.co.edge.jseq.Activation;
import th.co.edge.jseq.ActivationList;
import th.co.edge.jseq.Diagram;

/**
 * Can be used to transform an activation list (i.e. a list of method calls)
 * into the sdedit syntax for sequence diagrams.
 * 
 * @author jacek.ratzinger
 *
 */
public class SdeditTextDiagram implements Diagram
{
	private static final String NEW_LINE = "\n";
	private ActivationList activationList;
    
	/**
	 * Create an instance of the textual description for sequence diagrams.
	 * 
	 * @param activationList
	 *            The list of activations (i.e. message flows between objects).
	 */
    public SdeditTextDiagram(ActivationList activationList) {
    	this.activationList = activationList;
    }

    /**
     * Saves the textual representation suitable for
     * the Quick Sequence Diagram Editor (sdedit).
     * 
     * @param file
     *            The txt file to write the textual sequence diagram to.
     * 
     * @throws IOException
     *            if any problems occurs during file access.
     */
    public void save(File file) throws IOException {
    	writeToFile(activationList, file);
    }
    
    private void writeToFile(ActivationList activationList, File file) throws IOException
    {
        FileWriter writer = null;
        try
        {
        	writer = new FileWriter(file);
        	
        	writer.write("actor:Actor");
        	writer.write(NEW_LINE);
        	writeObjects(activationList, writer, new ArrayList<String>());
        	
			for (Activation activation : activationList) {
				writeActivation(activation, 1, writer);
				writer.write(NEW_LINE);
			}
        }
        finally
        {
            if(writer != null)
                try
                {
                    writer.close();
                }
                catch(IOException e) { }
        }
    }
    
    private void writeObjects(ActivationList activationList, FileWriter writer, List<String> uniqueClassNames) throws IOException {
    	for (Activation activation : activationList) {
    		if ("<init>".equals(activation.getMethod().name())) continue;
			if ("<clinit>".equals(activation.getMethod().name())) continue;
    		if (uniqueClassNames.contains(getClassName(activation)) == false) {
    			uniqueClassNames.add(getClassName(activation));
	    		writer.write(getClassName(activation).toLowerCase());
	    		writer.write(":");
    			writer.write(getClassName(activation));
    			writer.write("[a]");
	    		writer.write(NEW_LINE);
    		}
    		
    		writeObjects(activation.getNestedActivations(), writer, uniqueClassNames);
    	}
    }

	private String getClassName(Activation activation) {
		String qualifiedClassName = activation.getClassName();
		int lastDot = qualifiedClassName.lastIndexOf('.');
		String result = qualifiedClassName.substring(lastDot + 1);
		if (result.contains("$")) {
			int lastInnerClassSeperator = result.lastIndexOf('$');
			result = result.substring(lastInnerClassSeperator + 1);
		}
		return result;
	}

	private void writeActivation(Activation activation, int nestingLevel, FileWriter writer) throws IOException {
		if (activation.getParent() != null) {
			String indent = "";
			for (int i = 0; i < nestingLevel; i++) {
				indent += "  ";
			}
			writer.write(indent);
	
			writer.write(getClassName(activation.getParent()).toLowerCase());
			writer.write("[" + getMethodName(activation.getParent()) + (nestingLevel - 1) + "]");
		} else {
			nestingLevel--;
			writer.write(NEW_LINE);
			writer.write("actor");
		}
		writer.write(":");
		writer.write(getClassName(activation).toLowerCase());
		writer.write("[" + getMethodName(activation) + nestingLevel + "]");
		writer.write(".");
		writer.write(getMethodName(activation));
		
		for (Activation nestedActivation : activation.getNestedActivations()) {
			if ("<init>".equals(nestedActivation.getMethod().name())) continue;
			if ("<clinit>".equals(nestedActivation.getMethod().name())) continue;
			writer.write(NEW_LINE);
			writeActivation(nestedActivation, nestingLevel + 1, writer);
		}
	}

	private String getMethodName(Activation activation) {
		return activation.getMethod().name();
	}

    public String toString()
    {
        return activationList.toString();
    }
}